/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: EjbJarExternLibraries.java 3243 2008-05-15 14:39:38Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.EjbJarDeployableMetadataConfigurator;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.ArchiveScanner;
import org.ow2.util.scan.api.ArchiveScannerFactory;
import org.ow2.util.scan.api.ScanException;

/**
 * Default implementation.
 * @param <E> ejb jar deployable type
 * @author Gael Lalire
 */
public class EjbJarExternLibraries<E extends EJBDeployable<E>> {

    /**
     * A logger.
     */
    private static final Log LOGGER = LogFactory.getLog(EjbJarExternLibraries.class);

    /**
     * Associate a class name to its metadata.
     */
    private Map<String, IEjbJarClassMetadata<?>> classMetadata;

    /**
     * Associate an archive scanner to its metadata.
     */
    private Map<ArchiveScanner, IEjbJarDeployableMetadata<?>> scanners;

    /**
     * Construct an extern library.
     * @param archiveScannerFactory the archive scanner factory
     * @param archiveList the archive list
     */
    public EjbJarExternLibraries(final ArchiveScannerFactory archiveScannerFactory, final List<IArchive> archiveList) {
        for (IArchive archive : archiveList) {
            // no lib deployable so call getDeployable will return null
            EjbJarDeployableMetadataConfigurator<E> ejbJarArchiveMetadataConfigurator =
                new EjbJarDeployableMetadataConfigurator<E>(null);
            ArchiveScanner archiveScanner = archiveScannerFactory.createArchiveScanner(archive);
            archiveScanner.addArchiveConfigurator(ejbJarArchiveMetadataConfigurator);
            scanners.put(archiveScanner, ejbJarArchiveMetadataConfigurator.getEjbJarArchiveMetadata());
        }
    }

    /**
     * Try to find extern metadata.
     * @param className the class name
     * @return the metadata or null
     */
    public IEjbJarClassMetadata<?> findMetadata(final String className) {
        IEjbJarClassMetadata<?> ejbJarClassMetadata = classMetadata.get(className);
        if (ejbJarClassMetadata != null) {
            return ejbJarClassMetadata;
        }
        List<String> classNameList = new ArrayList<String>();
        classNameList.add(className);
        Iterator<Entry<ArchiveScanner, IEjbJarDeployableMetadata<?>>> entries = scanners.entrySet().iterator();
        while (ejbJarClassMetadata == null && entries.hasNext()) {
            Entry<ArchiveScanner, IEjbJarDeployableMetadata<?>> entry = entries.next();
            try {
                // TODO gaellalire : add method scanClassName (instead of scan collection)
                // which returns a boolean
                entry.getKey().scanCollection(classNameList);
                ejbJarClassMetadata = entry.getValue().getEjbJarClassMetadata(className);
            } catch (ScanException e) {
                LOGGER.debug("error when scan collection", e);
            }
        }
        if (ejbJarClassMetadata != null) {
            classMetadata.put(className, ejbJarClassMetadata);
        }
        return ejbJarClassMetadata;
    }

    /**
     * Stop all archive scanner.
     */
    public void close() {
        for (ArchiveScanner archiveScanner : scanners.keySet()) {
            archiveScanner.endScan();
        }
    }


}
